@@ -1,6 +1,6 @@
 /*
 ***************************************************************************
-* Copyright (C) 1999-2009, International Business Machines Corporation
+* Copyright (C) 1999-2011, International Business Machines Corporation
 * and others. All Rights Reserved.
 ***************************************************************************
 *   Date        Name        Description
@@ -476,7 +476,7 @@ public:
      * @param uset a USet (the ICU plain C type for UnicodeSet)
      * @return the corresponding UnicodeSet pointer.
      *
-     * @draft ICU 4.2
+     * @stable ICU 4.2
      */
     inline static UnicodeSet *fromUSet(USet *uset);
 
@@ -486,7 +486,7 @@ public:
      * @param uset a const USet (the ICU plain C type for UnicodeSet)
      * @return the corresponding UnicodeSet pointer.
      *
-     * @draft ICU 4.2
+     * @stable ICU 4.2
      */
     inline static const UnicodeSet *fromUSet(const USet *uset);
     
@@ -495,7 +495,7 @@ public:
      * USet is the plain C type for UnicodeSet
      *
      * @return a USet pointer for this UnicodeSet
-     * @draft ICU 4.2
+     * @stable ICU 4.2
      */
     inline USet *toUSet();
 
@@ -505,7 +505,7 @@ public:
      * USet is the plain C type for UnicodeSet
      *
      * @return a const USet pointer for this UnicodeSet
-     * @draft ICU 4.2
+     * @stable ICU 4.2
      */
     inline const USet * toUSet() const;
 
@@ -575,8 +575,8 @@ public:
 
     /**
      * Modifies this set to represent the set specified by the given
-     * pattern, optionally ignoring white space.  See the class
-     * description for the syntax of the pattern language.
+     * pattern, ignoring Unicode Pattern_White_Space characters.
+     * See the class description for the syntax of the pattern language.
      * A frozen set will not be modified.
      * @param pattern a string specifying what characters are in the set
      * @param status returns <code>U_ILLEGAL_ARGUMENT_ERROR</code> if the pattern
@@ -590,8 +590,8 @@ public:
 
     /**
      * Modifies this set to represent the set specified by the given
-     * pattern, optionally ignoring white space.  See the class
-     * description for the syntax of the pattern language.
+     * pattern, optionally ignoring Unicode Pattern_White_Space characters.
+     * See the class description for the syntax of the pattern language.
      * A frozen set will not be modified.
      * @param pattern a string specifying what characters are in the set
      * @param options bitmask for options to apply to the pattern.
@@ -861,6 +861,20 @@ public:
      */
     int32_t span(const UChar *s, int32_t length, USetSpanCondition spanCondition) const;
 
+    /**
+     * Returns the end of the substring of the input string according to the USetSpanCondition.
+     * Same as <code>start+span(s.getBuffer()+start, s.length()-start, spanCondition)</code>
+     * after pinning start to 0<=start<=s.length().
+     * @param s the string
+     * @param start the start index in the string for the span operation
+     * @param spanCondition specifies the containment condition
+     * @return the exclusive end of the substring according to the spanCondition;
+     *         the substring s.tempSubStringBetween(start, end) fulfills the spanCondition
+     * @stable ICU 4.4
+     * @see USetSpanCondition
+     */
+    inline int32_t span(const UnicodeString &s, int32_t start, USetSpanCondition spanCondition) const;
+
     /**
      * Returns the start of the trailing substring of the input string which
      * consists only of characters and strings that are contained in this set
@@ -880,6 +894,21 @@ public:
      */
     int32_t spanBack(const UChar *s, int32_t length, USetSpanCondition spanCondition) const;
 
+    /**
+     * Returns the start of the substring of the input string according to the USetSpanCondition.
+     * Same as <code>spanBack(s.getBuffer(), limit, spanCondition)</code>
+     * after pinning limit to 0<=end<=s.length().
+     * @param s the string
+     * @param limit the exclusive-end index in the string for the span operation
+     *              (use s.length() or INT32_MAX for spanning back from the end of the string)
+     * @param spanCondition specifies the containment condition
+     * @return the start of the substring according to the spanCondition;
+     *         the substring s.tempSubStringBetween(start, limit) fulfills the spanCondition
+     * @stable ICU 4.4
+     * @see USetSpanCondition
+     */
+    inline int32_t spanBack(const UnicodeString &s, int32_t limit, USetSpanCondition spanCondition) const;
+
     /**
      * Returns the length of the initial substring of the input string which
      * consists only of characters and strings that are contained in this set
@@ -945,6 +974,7 @@ private:
      * @param limit the limit offset for matching, either last+1 in
      * the forward direction, or last-1 in the reverse direction,
      * where last is the index of the last character to match.
+     * @param s
      * @return If part of s matches up to the limit, return |limit -
      * start|.  If all of s matches before reaching the limit, return
      * s.length().  If there is a mismatch between s and text, return
@@ -1293,7 +1323,7 @@ public:
      * Currently only the USET_CASE bit is supported.  Any undefined bits
      * are ignored.
      * @return a reference to this set.
-     * @draft ICU 4.2
+     * @stable ICU 4.2
      */
     UnicodeSet& closeOver(int32_t attribute);
 
@@ -1301,7 +1331,7 @@ public:
      * Remove all strings from this set.
      *
      * @return a reference to this set.
-     * @draft ICU 4.2
+     * @stable ICU 4.2
      */
     virtual UnicodeSet &removeAllStrings();
 
@@ -1511,8 +1541,8 @@ private:
      * \\p{foo} \\P{foo}  - white space not allowed within "\\p" or "\\P"
      * \\N{name}         - white space not allowed within "\\N"
      *
-     * Other than the above restrictions, white space is ignored.  Case
-     * is ignored except in "\\p" and "\\P" and "\\N".  In 'name' leading
+     * Other than the above restrictions, Unicode Pattern_White_Space characters are ignored.
+     * Case is ignored except in "\\p" and "\\P" and "\\N".  In 'name' leading
      * and trailing space is deleted, and internal runs of whitespace
      * are collapsed to a single space.
      *
@@ -1533,6 +1563,7 @@ private:
      * On return, the position after the last character parsed, that is,
      * the locations marked '%'.  If the parse fails, ppos is returned
      * unchanged.
+     * @param ec status
      * @return a reference to this.
      */
     UnicodeSet& applyPropertyPattern(const UnicodeString& pattern,
@@ -1619,6 +1650,26 @@ inline const USet *UnicodeSet::toUSet() const {
     return reinterpret_cast<const USet *>(this);
 }
 
+inline int32_t UnicodeSet::span(const UnicodeString &s, int32_t start, USetSpanCondition spanCondition) const {
+    int32_t sLength=s.length();
+    if(start<0) {
+        start=0;
+    } else if(start>sLength) {
+        start=sLength;
+    }
+    return start+span(s.getBuffer()+start, sLength-start, spanCondition);
+}
+
+inline int32_t UnicodeSet::spanBack(const UnicodeString &s, int32_t limit, USetSpanCondition spanCondition) const {
+    int32_t sLength=s.length();
+    if(limit<0) {
+        limit=0;
+    } else if(limit>sLength) {
+        limit=sLength;
+    }
+    return spanBack(s.getBuffer(), limit, spanCondition);
+}
+
 U_NAMESPACE_END
 
 #endif
